@@ -1,19 +1,25 @@
 mod builder;
+mod token;
+
+use token::{Authorization, AuthorizationKind, Token, TokenResponse};
 
 pub use builder::OsuBuilder;
+pub use token::Scope;
 
-use crate::{error::OsuError, model::GameMode, ratelimiter::Ratelimiter, request::*, OsuResult};
+use crate::{
+    error::OsuError, model::GameMode, prelude::Username, ratelimiter::Ratelimiter, request::*,
+    OsuResult,
+};
 
 use bytes::Bytes;
-// use reqwest::{header::HeaderValue, multipart::Form, Client, Method, Response, StatusCode};
 use hyper::{
     client::{Client as HyperClient, HttpConnector},
     header::{HeaderValue, ACCEPT, AUTHORIZATION, CONTENT_LENGTH, CONTENT_TYPE, USER_AGENT},
     Body, Method, Request as HyperRequest, Response, StatusCode,
 };
-type HttpsConnector<T> = hyper_rustls::HttpsConnector<T>;
-use serde::{de::DeserializeOwned, Deserialize};
-use std::{ops::Drop, sync::Arc, time::Duration};
+use hyper_rustls::HttpsConnector;
+use serde::de::DeserializeOwned;
+use std::{fmt::Write, ops::Drop, sync::Arc, time::Duration};
 use tokio::sync::{oneshot::Sender, RwLock};
 use url::Url;
 
@@ -34,6 +40,7 @@ pub struct Osu {
     pub(crate) cache: Arc<DashMap<Username, u32>>,
     #[cfg(feature = "metrics")]
     pub(crate) metrics: Arc<Metrics>,
+    token_loop_tx: Option<Sender<()>>,
 }
 
 impl Osu {
@@ -71,10 +78,22 @@ impl Osu {
     /// have these options filled: `legacy_thread_url`, `ratings`,
     /// `ranked_date` (if not unranked) and `submitted_date` (if submitted).
     #[inline]
-    pub fn beatmap(&self) -> GetBeatmap {
+    pub fn beatmap(&self) -> GetBeatmap<'_> {
         GetBeatmap::new(self)
     }
 
+    /// Get a vec of at most 50 [`BeatmapCompact`](crate::model::beatmap::BeatmapCompact)s.
+    ///
+    /// The contained maps will have these options filled: `mapset`,
+    /// `fail_times`, and `max_combo` (if available for mode).
+    #[inline]
+    pub fn beatmaps<I>(&self, map_ids: I) -> GetBeatmaps<'_>
+    where
+        I: IntoIterator<Item = u32>,
+    {
+        GetBeatmaps::new(self, map_ids)
+    }
+
     /// Get a vec of [`Score`](crate::model::score::Score).
     ///
     /// The contained scores will have the following options filled:
@@ -83,7 +102,7 @@ impl Osu {
     /// The scores' contained [`UserCompact`](crate::model::user::UserCompact)
     /// will have the `country` and `cover` options filled.
     #[inline]
-    pub fn beatmap_scores(&self, map_id: u32) -> GetBeatmapScores {
+    pub fn beatmap_scores(&self, map_id: u32) -> GetBeatmapScores<'_> {
         GetBeatmapScores::new(self, map_id)
     }
 
@@ -93,7 +112,7 @@ impl Osu {
     /// `map` and `user` options filled.
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn beatmap_user_score(&self, map_id: u32, user_id: u32) -> GetBeatmapUserScore {
+    pub fn beatmap_user_score(&self, map_id: u32, user_id: u32) -> GetBeatmapUserScore<'_> {
         GetBeatmapUserScore::new(self, map_id, user_id)
     }
 
@@ -107,7 +126,7 @@ impl Osu {
         &self,
         map_id: u32,
         user_id: impl Into<UserId>,
-    ) -> GetBeatmapUserScore {
+    ) -> GetBeatmapUserScore<'_> {
         GetBeatmapUserScore::new(self, map_id, user_id.into())
     }
 
@@ -122,14 +141,14 @@ impl Osu {
     /// will contain `Some` in `fail_times`, `max_combo`
     /// (if available for mode), and `deleted_at` (if deleted).
     #[inline]
-    pub fn beatmapset(&self, mapset_id: u32) -> GetBeatmapset {
+    pub fn beatmapset(&self, mapset_id: u32) -> GetBeatmapset<'_> {
         GetBeatmapset::new(self, mapset_id)
     }
 
     /// Get a [`BeatmapsetEvents`](crate::model::beatmap::BeatmapsetEvents)
     /// struct containing the most recent mapset events.
     #[inline]
-    pub fn beatmapset_events(&self) -> GetBeatmapsetEvents {
+    pub fn beatmapset_events(&self) -> GetBeatmapsetEvents<'_> {
         GetBeatmapsetEvents::new(self)
     }
 
@@ -163,14 +182,14 @@ impl Osu {
     /// let query = "status=loved artist=camellia stars>8";
     /// ```
     #[inline]
-    pub fn beatmapset_search(&self) -> GetBeatmapsetSearch {
+    pub fn beatmapset_search(&self) -> GetBeatmapsetSearch<'_> {
         GetBeatmapsetSearch::new(self)
     }
 
     /// Get a list of comments and their replies up to two levels deep
     /// in form of a [`CommentBundle`](crate::model::comments::CommentBundle) .
     #[inline]
-    pub fn comments(&self) -> GetComments {
+    pub fn comments(&self) -> GetComments<'_> {
         GetComments::new(self)
     }
 
@@ -186,7 +205,7 @@ impl Osu {
     /// The statistics vector is ordered by `ranked_score`.
     /// The `user` option is filled.
     #[inline]
-    pub fn chart_rankings(&self, mode: GameMode) -> GetChartRankings {
+    pub fn chart_rankings(&self, mode: GameMode) -> GetChartRankings<'_> {
         GetChartRankings::new(self, mode)
     }
 
@@ -194,13 +213,13 @@ impl Osu {
     /// containing a vec of [`CountryRanking`](crate::model::ranking::CountryRanking)s
     /// which will be sorted by the country's total pp.
     #[inline]
-    pub fn country_rankings(&self, mode: GameMode) -> GetCountryRankings {
+    pub fn country_rankings(&self, mode: GameMode) -> GetCountryRankings<'_> {
         GetCountryRankings::new(self, mode)
     }
 
     /// Get a [`ForumPosts`](crate::model::forum::ForumPosts) struct for a forum topic
     #[inline]
-    pub fn forum_posts(&self, topic_id: u64) -> GetForumPosts {
+    pub fn forum_posts(&self, topic_id: u64) -> GetForumPosts<'_> {
         GetForumPosts::new(self, topic_id)
     }
 
@@ -208,7 +227,7 @@ impl Osu {
     /// [`KudosuHistory`](crate::model::kudosu::KudosuHistory).
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn kudosu(&self, user_id: u32) -> GetUserKudosu {
+    pub fn kudosu(&self, user_id: u32) -> GetUserKudosu<'_> {
         GetUserKudosu::new(self, user_id)
     }
 
@@ -216,65 +235,45 @@ impl Osu {
     /// [`KudosuHistory`](crate::model::kudosu::KudosuHistory).
     #[cfg(feature = "cache")]
     #[inline]
-    pub fn kudosu(&self, user_id: impl Into<UserId>) -> GetUserKudosu {
+    pub fn kudosu(&self, user_id: impl Into<UserId>) -> GetUserKudosu<'_> {
         GetUserKudosu::new(self, user_id.into())
     }
 
-    // TODO: Documentation
-    #[deprecated = "The API currently doesn't allow this endpoint for public use"]
-    #[inline]
-    pub fn multiplayer_score(
-        &self,
-        room: u32,
-        playlist: u32,
-        score_id: u32,
-    ) -> GetMultiplayerScore {
-        GetMultiplayerScore::new(self, room, playlist, score_id)
-    }
-
-    // TODO: Documentation
-    #[deprecated = "The API currently doesn't allow this endpoint for public use"]
-    #[inline]
-    pub fn multiplayer_scores(&self, room: u32, playlist: u32) -> GetMultiplayerScores {
-        GetMultiplayerScores::new(self, room, playlist)
-    }
-
-    // TODO: Documentation
-    #[deprecated = "The API currently doesn't allow this endpoint for public use"]
-    #[inline]
-    pub fn multiplayer_user_highscore(
-        &self,
-        room: u32,
-        playlist: u32,
-        user_id: u32,
-    ) -> GetMultiplayerUserHighScore {
-        GetMultiplayerUserHighScore::new(self, room, playlist, user_id)
-    }
-
     /// Get [`News`](crate::model::news::News).
     #[inline]
-    pub fn news(&self) -> GetNews {
+    pub fn news(&self) -> GetNews<'_> {
         GetNews::new(self)
     }
 
     /// Get an [`OsuMatch`](crate::model::matches::OsuMatch).
     #[inline]
-    pub fn osu_match(&self, match_id: u32) -> GetMatch {
+    pub fn osu_match(&self, match_id: u32) -> GetMatch<'_> {
         GetMatch::new(self, match_id)
     }
 
     /// Get a [`MatchList`](crate::model::matches::MatchList) containing all
     /// currently open multiplayer lobbies.
     #[inline]
-    pub fn osu_matches(&self) -> GetMatches {
+    pub fn osu_matches(&self) -> GetMatches<'_> {
         GetMatches::new(self)
     }
 
+    /// Get the [`User`](crate::model::user::User) of the authenticated user.
+    ///
+    /// Note that the client has to be initialized with the `identify` scope
+    /// through the OAuth process in order for this endpoint to not return an error.
+    ///
+    /// See [`OsuBuilder::with_authorization`](crate::OsuBuilder::with_authorization).
+    #[inline]
+    pub fn own_data(&self) -> GetOwnData<'_> {
+        GetOwnData::new(self)
+    }
+
     /// Get a [`Rankings`](crate::model::ranking::Rankings) struct whose
     /// [`UserCompact`](crate::model::user::UserCompact)s are sorted
     /// by their pp, i.e. the current pp leaderboard.
     #[inline]
-    pub fn performance_rankings(&self, mode: GameMode) -> GetPerformanceRankings {
+    pub fn performance_rankings(&self, mode: GameMode) -> GetPerformanceRankings<'_> {
         GetPerformanceRankings::new(self, mode)
     }
 
@@ -282,7 +281,7 @@ impl Osu {
     /// [`RecentEvent`](crate::model::recent_event::RecentEvent)s.
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn recent_events(&self, user_id: u32) -> GetRecentEvents {
+    pub fn recent_events(&self, user_id: u32) -> GetRecentEvents<'_> {
         GetRecentEvents::new(self, user_id)
     }
 
@@ -290,27 +289,39 @@ impl Osu {
     /// [`RecentEvent`](crate::model::recent_event::RecentEvent)s.
     #[cfg(feature = "cache")]
     #[inline]
-    pub fn recent_events(&self, user_id: impl Into<UserId>) -> GetRecentEvents {
+    pub fn recent_events(&self, user_id: impl Into<UserId>) -> GetRecentEvents<'_> {
         GetRecentEvents::new(self, user_id.into())
     }
 
+    /// Get a [`Score`](crate::model::score::Score) struct.
+    ///
+    /// The contained score will have the following options filled:
+    /// `map` (will contain `checksum` and `max_combo`), `mapset`
+    /// (will contain `artist_unicode` and `title_unicode`), `pp`
+    /// (if ranked), `rank_global` (if on leaderboard map) and `user`
+    /// (will contain `last_visited`, `country`, `cover` and `groups`)
+    #[inline]
+    pub fn score(&self, score_id: u64, mode: GameMode) -> GetScore<'_> {
+        GetScore::new(self, score_id, mode)
+    }
+
     /// Get a [`Rankings`](crate::model::ranking::Rankings) struct whose
     /// [`UserCompact`](crate::model::user::UserCompact)s are sorted
     /// by their ranked score, i.e. the current ranked score leaderboard.
     #[inline]
-    pub fn score_rankings(&self, mode: GameMode) -> GetScoreRankings {
+    pub fn score_rankings(&self, mode: GameMode) -> GetScoreRankings<'_> {
         GetScoreRankings::new(self, mode)
     }
 
     /// Get [`SeasonalBackgrounds`](crate::model::seasonal_backgrounds::SeasonalBackgrounds).
     #[inline]
-    pub fn seasonal_backgrounds(&self) -> GetSeasonalBackgrounds {
+    pub fn seasonal_backgrounds(&self) -> GetSeasonalBackgrounds<'_> {
         GetSeasonalBackgrounds::new(self)
     }
 
     /// Get the vec of [`Spotlight`](crate::model::ranking::Spotlight).
     #[inline]
-    pub fn spotlights(&self) -> GetSpotlights {
+    pub fn spotlights(&self) -> GetSpotlights<'_> {
         GetSpotlights::new(self)
     }
 
@@ -325,7 +336,7 @@ impl Osu {
     ///
     /// All other options will be filled.
     #[inline]
-    pub fn user(&self, user_id: impl Into<UserId>) -> GetUser {
+    pub fn user(&self, user_id: impl Into<UserId>) -> GetUser<'_> {
         GetUser::new(self, user_id)
     }
 
@@ -345,7 +356,7 @@ impl Osu {
     /// All options of the contained [`Beatmap`](crate::model::beatmap::Beatmap)s will be `None`.
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn user_beatmapsets(&self, user_id: u32) -> GetUserBeatmapsets {
+    pub fn user_beatmapsets(&self, user_id: u32) -> GetUserBeatmapsets<'_> {
         GetUserBeatmapsets::new(self, user_id)
     }
 
@@ -356,7 +367,7 @@ impl Osu {
     /// All options of the contained [`Beatmap`](crate::model::beatmap::Beatmap)s will be `None`.
     #[cfg(feature = "cache")]
     #[inline]
-    pub fn user_beatmapsets(&self, user_id: impl Into<UserId>) -> GetUserBeatmapsets {
+    pub fn user_beatmapsets(&self, user_id: impl Into<UserId>) -> GetUserBeatmapsets<'_> {
         GetUserBeatmapsets::new(self, user_id.into())
     }
 
@@ -367,10 +378,10 @@ impl Osu {
     ///
     /// ## Limit
     ///
-    /// The API provides at most 51 results per requests, 100 in total.
+    /// The API provides at most 100 results, defaults to 5.
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn user_most_played(&self, user_id: u32) -> GetUserMostPlayed {
+    pub fn user_most_played(&self, user_id: u32) -> GetUserMostPlayed<'_> {
         GetUserMostPlayed::new(self, user_id)
     }
 
@@ -381,19 +392,20 @@ impl Osu {
     ///
     /// ## Limit
     ///
-    /// The API provides at most 51 results per requests, 100 in total.
+    /// The API provides at most 100 results, defaults to 5.
     #[cfg(feature = "cache")]
     #[inline]
-    pub fn user_most_played(&self, user_id: impl Into<UserId>) -> GetUserMostPlayed {
+    pub fn user_most_played(&self, user_id: impl Into<UserId>) -> GetUserMostPlayed<'_> {
         GetUserMostPlayed::new(self, user_id.into())
     }
 
-    /// Get either top, global firsts, or recent scores of a user,
+    /// Get either top, global firsts, pinned, or recent scores of a user,
     /// i.e. a vec of [`Score`](crate::model::score::Score).
     ///
     /// If no score type is specified by either
     /// [`best`](crate::request::GetUserScores::best),
     /// [`firsts`](crate::request::GetUserScores::firsts),
+    /// [`pinned`](crate::request::GetUserScores::pinned),
     /// or [`recent`](crate::request::GetUserScores::recent), it defaults to `best`.
     ///
     /// The resulting scores will have these options filled: `map`, `mapset`, `pp`, `user`
@@ -406,7 +418,7 @@ impl Osu {
     ///
     /// ## Note
     ///
-    /// - The API provides at most 51 results per requests, 100 in total.
+    /// - The API provides at most 100 results per requests and defaults to 5.
     /// - For the `recent` score type, failed score are excluded by default.
     /// Use [`include_fails`](crate::request::GetUserScores::include_fails)
     /// to include them.
@@ -414,16 +426,17 @@ impl Osu {
     /// is not loved.
     #[cfg(not(feature = "cache"))]
     #[inline]
-    pub fn user_scores(&self, user_id: u32) -> GetUserScores {
+    pub fn user_scores(&self, user_id: u32) -> GetUserScores<'_> {
         GetUserScores::new(self, user_id)
     }
 
-    /// Get either top, global firsts, or recent scores of a user,
+    /// Get either top, global firsts, pinned, or recent scores of a user,
     /// i.e. a vec of [`Score`](crate::model::score::Score).
     ///
     /// If no score type is specified by either
     /// [`best`](crate::request::GetUserScores::best),
     /// [`firsts`](crate::request::GetUserScores::firsts),
+    /// [`pinned`](crate::request::GetUserScores::pinned),
     /// or [`recent`](crate::request::GetUserScores::recent), it defaults to `best`.
     ///
     /// The resulting scores will have these options filled: `map`, `mapset`, `pp`, `user`
@@ -436,7 +449,7 @@ impl Osu {
     ///
     /// ## Note
     ///
-    /// - The API provides at most 51 results per requests, 100 in total.
+    /// - The API provides at most 100 results per requests and defaults to 5.
     /// - For the `recent` score type, failed score are excluded by default.
     /// Use [`include_fails`](crate::request::GetUserScores::include_fails)
     /// to include them.
@@ -444,14 +457,14 @@ impl Osu {
     /// is not loved.
     #[cfg(feature = "cache")]
     #[inline]
-    pub fn user_scores(&self, user_id: impl Into<UserId>) -> GetUserScores {
+    pub fn user_scores(&self, user_id: impl Into<UserId>) -> GetUserScores<'_> {
         GetUserScores::new(self, user_id.into())
     }
 
     /// Get a vec of [`UserCompact`](crate::model::user::UserCompact).
     #[deprecated = "The API currently doesn't allow this endpoint for public use"]
     #[inline]
-    pub fn users(&self, user_ids: &[u32]) -> GetUsers {
+    pub fn users(&self, user_ids: &[u32]) -> GetUsers<'_> {
         GetUsers::new(self, user_ids)
     }
 
@@ -459,7 +472,7 @@ impl Osu {
     ///
     /// `locale` adjusts the language, e.g. `en` for english, `de` for german, ...
     #[inline]
-    pub fn wiki(&self, locale: impl Into<String>) -> GetWikiPage {
+    pub fn wiki(&self, locale: impl Into<String>) -> GetWikiPage<'_> {
         GetWikiPage::new(self, locale)
     }
 
@@ -484,14 +497,22 @@ impl Osu {
     }
 }
 
+impl Drop for Osu {
+    fn drop(&mut self) {
+        if let Some(tx) = self.token_loop_tx.take() {
+            let _ = tx.send(());
+        }
+    }
+}
+
 pub(crate) struct OsuRef {
-    pub(crate) client_id: u64,
-    pub(crate) client_secret: String,
-    pub(crate) http: HyperClient<HttpsConnector<HttpConnector>, Body>,
-    pub(crate) timeout: Duration,
-    pub(crate) ratelimiter: Ratelimiter,
-    pub(crate) token: RwLock<Option<String>>,
-    pub(crate) token_loop_tx: Option<Sender<()>>,
+    client_id: u64,
+    client_secret: String,
+    http: HyperClient<HttpsConnector<HttpConnector>, Body>,
+    timeout: Duration,
+    ratelimiter: Ratelimiter,
+    auth_kind: AuthorizationKind,
+    token: RwLock<Token>,
 }
 
 static MY_USER_AGENT: &str = concat!(
@@ -505,139 +526,56 @@ static MY_USER_AGENT: &str = concat!(
 const APPLICATION_JSON: &str = "application/json";
 
 impl OsuRef {
-    pub(crate) async fn request_token(&self) -> OsuResult<Token> {
-        let data = format!(
-            r#"{{"client_id":{},"client_secret":"{}","grant_type":"client_credentials","scope":"public"}}"#,
-            self.client_id, self.client_secret
+    async fn request_token(&self) -> OsuResult<TokenResponse> {
+        let mut data = format!(
+            r#"{{"client_id":{},"client_secret":"{}","grant_type":""#,
+            self.client_id, self.client_secret,
         );
 
-        let bytes = data.into_bytes();
+        let _ = match &self.auth_kind {
+            AuthorizationKind::Client(scope) => {
+                write!(data, r#"client_credentials","scope":"{}"}}"#, scope)
+            }
+            AuthorizationKind::User(auth) => match &self.token.read().await.refresh {
+                Some(refresh) => {
+                    write!(data, r#"refresh_token","refresh_token":"{}"}}"#, refresh)
+                }
+                None => {
+                    write!(
+                        data,
+                        r#"authorization_code","redirect_uri":"{}","code":"{}"}}"#,
+                        auth.redirect_uri, auth.code,
+                    )
+                }
+            },
+        };
 
-        let user_agent = HeaderValue::from_static(MY_USER_AGENT);
+        let bytes = data.into_bytes();
         let url = "https://osu.ppy.sh/oauth/token";
 
         let req = HyperRequest::builder()
             .method(Method::POST)
             .uri(url)
-            .header(USER_AGENT, user_agent)
+            .header(USER_AGENT, MY_USER_AGENT)
             .header(ACCEPT, APPLICATION_JSON)
             .header(CONTENT_TYPE, APPLICATION_JSON)
             .header(CONTENT_LENGTH, bytes.len())
             .body(Body::from(bytes))?;
 
-        self.ratelimiter.await_access().await;
+        let resp = self.send_request(req).await?;
+        let bytes = self.handle_status(resp).await?;
 
-        let inner = self.http.request(req);
-        let fut = tokio::time::timeout(self.timeout, inner);
-
-        let resp = fut
-            .await
-            .map_err(|_| OsuError::RequestTimeout)?
-            .map_err(|source| OsuError::Request { source })?;
-
-        let status = resp.status();
-
-        match status {
-            StatusCode::OK => {
-                let bytes = hyper::body::to_bytes(resp.into_body())
-                    .await
-                    .map_err(|source| OsuError::ChunkingResponse { source })?;
-
-                return serde_json::from_slice(&bytes).map_err(|source| {
-                    let body = String::from_utf8_lossy(&bytes).into();
-                    OsuError::Parsing { body, source }
-                });
-            }
-            StatusCode::SERVICE_UNAVAILABLE => {
-                let body = hyper::body::to_bytes(resp.into_body())
-                    .await
-                    .as_ref()
-                    .map(|bytes| String::from_utf8_lossy(bytes))
-                    .map(|text| text.into_owned())
-                    .ok();
-
-                return Err(OsuError::ServiceUnavailable(body));
-            }
-            StatusCode::TOO_MANY_REQUESTS => warn!("429 response: {:?}", resp),
-            _ => {}
-        }
-
-        let bytes = hyper::body::to_bytes(resp.into_body())
-            .await
-            .map_err(|source| OsuError::ChunkingResponse { source })?;
-
-        let body = String::from_utf8_lossy(bytes.as_ref()).into_owned();
-
-        let source = match serde_json::from_slice(&bytes) {
-            Ok(source) => source,
-            Err(source) => return Err(OsuError::Parsing { body, source }),
-        };
-
-        Err(OsuError::Response {
-            body,
-            source,
-            status,
-        })
+        parse_bytes(bytes)
     }
 
     pub(crate) async fn request<T: DeserializeOwned>(&self, req: Request) -> OsuResult<T> {
-        let bytes = self.request_bytes(req).await?;
+        let resp = self.raw(req).await?;
+        let bytes = self.handle_status(resp).await?;
 
         // let text = String::from_utf8_lossy(&bytes);
         // println!("Response:\n{}", text);
 
-        serde_json::from_slice(&bytes).map_err(|source| {
-            let body = String::from_utf8_lossy(&bytes).into();
-
-            OsuError::Parsing { body, source }
-        })
-    }
-
-    pub(crate) async fn request_bytes(&self, req: Request) -> OsuResult<Bytes> {
-        let resp = self.make_request(req).await?;
-
-        hyper::body::to_bytes(resp.into_body())
-            .await
-            .map_err(|source| OsuError::ChunkingResponse { source })
-    }
-
-    async fn make_request(&self, req: Request) -> OsuResult<Response<Body>> {
-        let resp = self.raw(req).await?;
-        let status = resp.status();
-
-        match status {
-            StatusCode::OK => return Ok(resp),
-            StatusCode::NOT_FOUND => return Err(OsuError::NotFound),
-            StatusCode::SERVICE_UNAVAILABLE => {
-                let body = hyper::body::to_bytes(resp.into_body())
-                    .await
-                    .as_ref()
-                    .map(|bytes| String::from_utf8_lossy(bytes))
-                    .map(|text| text.into_owned())
-                    .ok();
-
-                return Err(OsuError::ServiceUnavailable(body));
-            }
-            StatusCode::TOO_MANY_REQUESTS => warn!("429 response: {:?}", resp),
-            _ => {}
-        }
-
-        let bytes = hyper::body::to_bytes(resp.into_body())
-            .await
-            .map_err(|source| OsuError::ChunkingResponse { source })?;
-
-        let body = String::from_utf8_lossy(bytes.as_ref()).into_owned();
-
-        let source = match serde_json::from_slice(&bytes) {
-            Ok(source) => source,
-            Err(source) => return Err(OsuError::Parsing { body, source }),
-        };
-
-        Err(OsuError::Response {
-            body,
-            source,
-            status,
-        })
+        parse_bytes(bytes)
     }
 
     async fn raw(&self, req: Request) -> OsuResult<Response<Body>> {
@@ -651,45 +589,73 @@ impl OsuRef {
         let url = Url::parse(&url).map_err(|source| OsuError::Url { source, url })?;
         debug!("URL: {}", url);
 
-        let mut builder = if let Some(token) = self.token.read().await.as_ref() {
+        if let Some(token) = self.token.read().await.access.as_ref() {
             let value = HeaderValue::from_str(token)
                 .map_err(|source| OsuError::CreatingTokenHeader { source })?;
 
-            HyperRequest::builder()
-                .method(method.clone())
+            let req = HyperRequest::builder()
+                .method(method)
                 .uri(url.as_str())
                 .header(AUTHORIZATION, value)
+                .header(USER_AGENT, MY_USER_AGENT)
+                .header(ACCEPT, APPLICATION_JSON)
+                .header(CONTENT_LENGTH, 0)
+                .body(Body::empty())?;
+
+            self.send_request(req).await
         } else {
-            return Err(OsuError::NoToken);
-        };
-
-        builder = builder
-            .header(USER_AGENT, MY_USER_AGENT)
-            .header(ACCEPT, APPLICATION_JSON)
-            .header(CONTENT_LENGTH, 0);
+            Err(OsuError::NoToken)
+        }
+    }
 
+    async fn send_request(&self, req: HyperRequest<Body>) -> OsuResult<Response<Body>> {
         self.ratelimiter.await_access().await;
 
-        let inner = self.http.request(builder.body(Body::empty())?);
-
-        tokio::time::timeout(self.timeout, inner)
+        tokio::time::timeout(self.timeout, self.http.request(req))
             .await
             .map_err(|_| OsuError::RequestTimeout)?
             .map_err(|source| OsuError::Request { source })
     }
-}
 
-impl Drop for OsuRef {
-    fn drop(&mut self) {
-        if let Some(tx) = self.token_loop_tx.take() {
-            let _ = tx.send(());
+    async fn handle_status(&self, resp: Response<Body>) -> OsuResult<Bytes> {
+        let status = resp.status();
+
+        let bytes = hyper::body::to_bytes(resp.into_body())
+            .await
+            .map_err(|source| OsuError::ChunkingResponse { source })?;
+
+        match status {
+            StatusCode::OK => return Ok(bytes),
+            StatusCode::NOT_FOUND => return Err(OsuError::NotFound),
+            StatusCode::SERVICE_UNAVAILABLE => {
+                let body = String::from_utf8_lossy(&bytes).into_owned();
+
+                return Err(OsuError::ServiceUnavailable(body));
+            }
+            StatusCode::TOO_MANY_REQUESTS => warn!("Got a 429 response"),
+            _ => {}
         }
+
+        let body = String::from_utf8_lossy(&bytes).into_owned();
+
+        let source = match serde_json::from_slice(&bytes) {
+            Ok(source) => source,
+            Err(source) => return Err(OsuError::Parsing { body, source }),
+        };
+
+        Err(OsuError::Response {
+            body,
+            source,
+            status,
+        })
     }
 }
 
-#[derive(Deserialize)]
-pub(crate) struct Token {
-    token_type: String,
-    expires_in: u64,
-    access_token: String,
+#[inline]
+fn parse_bytes<T: DeserializeOwned>(bytes: Bytes) -> OsuResult<T> {
+    serde_json::from_slice(&bytes).map_err(|source| {
+        let body = String::from_utf8_lossy(&bytes).into_owned();
+
+        OsuError::Parsing { body, source }
+    })
 }
