@@ -9,13 +9,14 @@ use url::ParseError;
 /// The API response was of the form `{ "error": ... }`
 #[derive(Debug, Deserialize)]
 pub struct ApiError {
+    /// Error specified by the API
     pub error: Option<String>,
 }
 
 impl StdError for ApiError {}
 
 impl fmt::Display for ApiError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.error {
             Some(ref msg) => f.write_str(msg),
             None => f.write_str("empty error message"),
@@ -56,13 +57,17 @@ pub enum OsuError {
         status: StatusCode,
     },
     /// Temporal (?) downtime of the osu API
-    ServiceUnavailable(Option<String>),
+    ServiceUnavailable(String),
     /// The client's authentication is not sufficient for the endpoint
     UnavailableEndpoint,
     /// Failed to update token
     UpdateToken { source: Box<OsuError> },
     /// Failed to parse the URL for a request
-    Url { source: ParseError, url: String },
+    Url {
+        source: ParseError,
+        /// URL that was attempted to be parsed
+        url: String,
+    },
 }
 
 impl StdError for OsuError {
@@ -89,7 +94,7 @@ impl StdError for OsuError {
 }
 
 impl fmt::Display for OsuError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::BodyError { .. } => f.write_str("failed to create request body"),
             Self::BuilderMissingId => {
@@ -119,7 +124,7 @@ impl fmt::Display for OsuError {
             Self::ServiceUnavailable(body) => write!(
                 f,
                 "osu!api may be temporarily unavailable (received 503): {}",
-                body.as_deref().unwrap_or("error while parsing body")
+                body
             ),
             Self::UnavailableEndpoint => {
                 f.write_str("the endpoint is not available for the client's authorization level")
@@ -145,21 +150,30 @@ impl From<ParsingError> for OsuError {
 /// Failed some TryFrom parsing
 #[derive(Debug)]
 pub enum ParsingError {
+    /// Failed to parse a u8 into a [`Genre`](crate::model::beatmap::Genre)
     Genre(u8),
+    /// Failed to parse a String into a [`Grade`](crate::model::Grade)
     Grade(String),
+    /// Failed to parse a u8 into a [`Language`](crate::model::beatmap::Language)
     Language(u8),
+    /// Failed to parse a u32 into [`GameMods`](crate::model::GameMods)
     ModsU32(u32),
+    /// Failed to parse a String into [`GameMods`](crate::model::GameMods)
     ModsStr(String),
+    /// Failed to parse an i8 into a [`RankStatus`](crate::model::beatmap::RankStatus)
     RankStatus(i8),
+    /// Failed to parse a u8 into a [`ScoringType`](crate::model::matches::ScoringType)
     ScoringType(u8),
+    /// Failed to parse a u8 into a [`Team`](crate::model::matches::Team)
     Team(u8),
+    /// Failed to parse a u8 into a [`TeamType`](crate::model::matches::TeamType)
     TeamType(u8),
 }
 
 impl StdError for ParsingError {}
 
 impl fmt::Display for ParsingError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Genre(n) => write!(f, "failed to parse {} into Genre", n),
             Self::Grade(s) => write!(f, "failed to parse `{}` into Grade", s),
