@@ -14,13 +14,17 @@
 //! Unlike api v1, api v2 does not require an api key by users. Instead, it requires a client id and a client secret.
 //!
 //! To get those, you must register an application [here](https://osu.ppy.sh/home/account/edit#new-oauth-application).
-//! Since rosu-v2 only supports client credentials and not authorization code, the callback URL here does not matter.
+//! Unless you're interested in logging into the API through an osu! account, the callback URL here does not matter and can be left blank.
+//!
+//! If you went through the OAuth process for a user, you can provide the callback URL and received code
+//! when creating the client in order to make requests on behalf of the authenticated user.
 //!
 //! ## Endpoints
 //!
 //! The following endpoints are currently supported:
 //!
 //! - `beatmaps/lookup`: A specific beatmap including its beatmapset
+//! - `beatmaps`: Up to 50 beatmaps at once including their beatmapsets.
 //! - `beatmaps/{map_id}/scores`: The global score leaderboard for a beatmap
 //! - `beatmaps/{map_id}/scores/users/{user_id}`: The single top score of a user on a beatmap. Defaults to the play with the __max score__, not pp
 //! - `beatmapsets/{mapset_id}`: The beatmapset including all of its difficulty beatmaps
@@ -30,15 +34,17 @@
 //! - `forums/topics/{topic_id}`: A forum topic and its posts
 //! - `matches`: List of currently open multiplayer lobbies
 //! - `matches/{match_id}`: More specific data about a specific multiplayer lobby including participating players and occured events
+//! - `me[/{mode}]`: Detailed info about the authenticated user [in the specified mode] (requires OAuth)
 //! - `news`: Recent news
 //! - `rankings/{mode}/{ranking_type}`: The global leaderboard of either performance points, ranked score, countries, or a spotlight
 //! - `users/{user_id}/{recent_activity}`: List of a user's recent events like achieved medals, ranks on a beatmaps, username changes, supporter status updates, beatmapset status updates, ...
+//! - `scores/{mode}/{score_id}`: A specific score including its beatmap, beatmapset, and user
 //! - `seasonal-backgrounds`: List of seasonal backgrounds i.e. their URL and artists
 //! - `spotlights`: List of overviews of all spotlights
 //! - `users/{user_id}[/{mode}]`: Detailed info about a user [in the specified mode]
 //! - `users/{user_id}/{beatmapsets/{map_type}`: List of beatmapsets either created, favourited, or most played by the user
 //! - `users/{user_id}/kudosu`: A user's recent kudosu transfers
-//! - `users/{user_id}/scores/{score_type}`: Either top scores, recent scores, or global #1 scores of a user
+//! - `users/{user_id}/scores/{score_type}`: Either top, recent, pinned, or global #1 scores of a user
 //! - `wiki/{locale}[/{path}]`: The general wiki page or a specific topic if the path is specified
 //!
 //! The api itself provides a bunch more endpoints which are not yet implemented because they're really niche and/or missing any documentation.
@@ -73,7 +79,7 @@
 //!     // If you are fine with just providing user ids, consider disabling this feature.
 //!     let scores: Vec<Score> = osu.user_scores("peppy")
 //!         .mode(GameMode::STD)
-//!         .best() // top scores; alternatively .recent() or .firsts()
+//!         .best() // top scores; alternatively .recent(), .pinned(), or .firsts()
 //!         .offset(10)
 //!         .limit(5)
 //!         .await
@@ -106,13 +112,20 @@
 //! | `cache` | Cache username-user_id pairs so that usernames can be used on all user endpoints instead of only user ids | [dashmap](https://github.com/xacrimon/dashmap)
 //! | `metrics` | Provide a count of all request types the client makes with the function `Osu::metrics` returning a `prometheus::IntCounterVec` | [prometheus](https://github.com/tikv/rust-prometheus)
 
+// #![deny(missing_docs)] // TODO
+#![deny(clippy::all, nonstandard_style, rust_2018_idioms, unused, warnings)]
+
 mod client;
-pub mod error;
-pub mod model;
 mod ratelimiter;
-pub mod request;
 mod routing;
 
+/// rosu-specific errors
+pub mod error;
+/// All available data types provided by the api
+pub mod model;
+/// Requesting-structs that implement [`Future`](std::future::Future) for each endpoint
+pub mod request;
+
 #[cfg(feature = "metrics")]
 mod metrics;
 
@@ -124,18 +137,22 @@ extern crate log;
 /// `Result<_, OsuError>`
 pub type OsuResult<T> = Result<T, error::OsuError>;
 
+/// All types except requesting, stuffed into one module
 pub mod prelude {
     pub use crate::{
+        client::Scope,
         error::OsuError,
         model::{
-            beatmap::*, comments::*, forum::*, kudosu::*, matches::*, multiplayer::*, news::*,
-            ranking::*, recent_event::*, score::*, seasonal_backgrounds::*, user::*, wiki::*,
-            Cursor, GameMode, GameMods, Grade,
+            beatmap::*, comments::*, forum::*, kudosu::*, matches::*, news::*, ranking::*,
+            recent_event::*, score::*, seasonal_backgrounds::*, user::*, wiki::*, Cursor, GameMode,
+            GameMods, Grade,
         },
+        request::UserId,
         Osu, OsuBuilder, OsuResult,
     };
 
     pub use hyper::StatusCode;
+    pub use smallstr::SmallString;
 
     #[cfg(feature = "metrics")]
     pub use prometheus::IntCounterVec;
