@@ -1,5 +1,5 @@
 use super::{user::UserCompact, Cursor, GameMode};
-use crate::{request::GetUser, Osu, OsuResult};
+use crate::{prelude::Username, request::GetUser, Osu, OsuResult};
 
 use chrono::{DateTime, Utc};
 use serde::{
@@ -116,6 +116,13 @@ impl From<Beatmap> for BeatmapCompact {
     }
 }
 
+#[derive(Deserialize)]
+pub(crate) struct Beatmaps {
+    #[serde(rename = "beatmaps")]
+    pub(crate) maps: Vec<BeatmapCompact>,
+}
+
+/// Represents a beatmapset. This extends [`BeatmapsetCompact`] with additional attributes.
 #[derive(Clone, Debug, Deserialize, Serialize)]
 pub struct Beatmapset {
     pub artist: String,
@@ -129,10 +136,11 @@ pub struct Beatmapset {
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub converts: Option<Vec<Beatmap>>,
     pub covers: BeatmapsetCovers,
+    /// Username of the mapper at the time of beatmapset creation
     #[serde(default, rename = "user", skip_serializing_if = "Option::is_none")]
     pub creator: Option<UserCompact>,
     #[serde(rename = "creator")]
-    pub creator_name: String,
+    pub creator_name: Username,
     #[serde(rename = "user_id")]
     pub creator_id: u32,
     #[serde(
@@ -280,9 +288,10 @@ pub struct BeatmapsetCommentOwnerChange {
     pub version: String,
     pub new_user_id: u32,
     #[serde(rename = "new_user_username")]
-    pub new_username: String,
+    pub new_username: Username,
 }
 
+/// Represents a beatmapset.
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct BeatmapsetCompact {
     pub artist: String,
@@ -290,7 +299,7 @@ pub struct BeatmapsetCompact {
     pub artist_unicode: Option<String>,
     pub covers: BeatmapsetCovers,
     #[serde(rename = "creator")]
-    pub creator_name: String,
+    pub creator_name: Username,
     #[serde(rename = "user_id")]
     pub creator_id: u32,
     pub favourite_count: u32,
@@ -307,6 +316,7 @@ pub struct BeatmapsetCompact {
     pub playcount: u32,
     /// Full URL, i.e. `b.ppy.sh/preview/{mapset_id}.mp3`
     pub preview_url: String,
+    // TODO: Add ratings
     pub source: String,
     pub status: RankStatus,
     pub title: String,
@@ -360,9 +370,9 @@ pub struct BeatmapsetCovers {
     /// Same as `card` but larger
     #[serde(rename = "card@2x")]
     pub card_2x: String,
-    /// Tiny preview of full the
+    /// Tiny preview of the full background
     pub list: String,
-    /// Small preview of full the background background
+    /// Small preview of the full background
     #[serde(rename = "list@2x")]
     pub list_2x: String,
     /// Same as `cover` but much larger
@@ -606,7 +616,7 @@ struct SearchRankStatusVisitor;
 impl<'de> Visitor<'de> for SearchRankStatusVisitor {
     type Value = SearchRankStatus;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a rank status, `any`, or 9")
     }
 
@@ -714,7 +724,7 @@ struct BeatmapsetSearchParametersVisitor;
 impl<'de> Visitor<'de> for BeatmapsetSearchParametersVisitor {
     type Value = BeatmapsetSearchParameters;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a search struct")
     }
 
@@ -813,6 +823,7 @@ impl<'de> Deserialize<'de> for BeatmapsetSearchParameters {
 #[derive(Clone, Debug, PartialEq, Serialize)]
 pub struct BeatmapsetSearchResult {
     cursor: Option<Cursor>,
+    /// All mapsets of the current page
     #[serde(rename(serialize = "beatmapsets"))]
     pub mapsets: Vec<Beatmapset>,
     #[serde(rename(serialize = "search"))]
@@ -822,12 +833,15 @@ pub struct BeatmapsetSearchResult {
 }
 
 impl BeatmapsetSearchResult {
+    /// Returns whether there is a next page of search results,
+    /// retrievable via [`get_next`](BeatmapsetSearchResult::get_next).
     #[inline]
     pub fn has_more(&self) -> bool {
         self.cursor.is_some()
     }
 
-    /// If `has_more()` is true, the API can provide the next set of search results and this method will request them.
+    /// If [`has_more`](BeatmapsetSearchResult::has_more) is true, the API can provide
+    /// the next set of search results and this method will request them.
     /// Otherwise, this method returns `None`.
     pub async fn get_next(&self, osu: &Osu) -> Option<OsuResult<BeatmapsetSearchResult>> {
         let cursor = self.cursor.as_ref()?.to_owned();
@@ -872,7 +886,7 @@ struct BeatmapsetSearchResultVisitor;
 impl<'de> Visitor<'de> for BeatmapsetSearchResultVisitor {
     type Value = BeatmapsetSearchResult;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a BeatmapsetSearchResult struct")
     }
 
@@ -932,7 +946,7 @@ macro_rules! search_sort_enum {
         }
 
         impl fmt::Display for BeatmapsetSearchSort {
-            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                 match self {
                     $(Self::$variant => f.write_str($name),)+
                 }
@@ -1008,14 +1022,17 @@ pub struct BeatmapsetVote {
     pub score: u32,
 }
 
+/// All fields are optional but there's always at least one field returned.
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct FailTimes {
+    /// List of length 100
     #[serde(
         default,
         deserialize_with = "use_vec_option_visitor",
         skip_serializing_if = "Option::is_none"
     )]
     pub exit: Option<Vec<u32>>,
+    /// List of length 100
     #[serde(
         default,
         deserialize_with = "use_vec_option_visitor",
@@ -1033,8 +1050,8 @@ struct VecOptionVisitor;
 impl<'de> Visitor<'de> for VecOptionVisitor {
     type Value = Option<Vec<u32>>;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "null or a sequence of u32")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("null or a sequence of u32")
     }
 
     fn visit_some<D: Deserializer<'de>>(self, d: D) -> Result<Self::Value, D::Error> {
@@ -1058,8 +1075,8 @@ impl HundredU32Visitor {
 impl<'de> Visitor<'de> for HundredU32Visitor {
     type Value = Vec<u32>;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "a sequence of u32")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("a sequence of u32")
     }
 
     fn visit_seq<A: SeqAccess<'de>>(mut self, mut seq: A) -> Result<Self::Value, A::Error> {
@@ -1153,8 +1170,8 @@ struct DescriptionVisitor;
 impl<'de> Visitor<'de> for DescriptionVisitor {
     type Value = Option<String>;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "a string or a map containing a 'description' field")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("a string or a map containing a 'description' field")
     }
 
     fn visit_str<E: Error>(self, v: &str) -> Result<Self::Value, E> {
