@@ -77,6 +77,7 @@ pub enum MatchEvent {
 }
 
 impl MatchEvent {
+    /// Return the id of the event
     pub fn event_id(&self) -> u64 {
         match self {
             Self::Create { event_id, .. } => *event_id,
@@ -89,6 +90,7 @@ impl MatchEvent {
         }
     }
 
+    /// Return the timestamp of the event
     pub fn timestamp(&self) -> DateTime<Utc> {
         match self {
             Self::Create { timestamp, .. } => *timestamp,
@@ -101,6 +103,7 @@ impl MatchEvent {
         }
     }
 
+    /// Return the user id of the user associated with the event
     pub fn user_id(&self) -> Option<u32> {
         match self {
             Self::Create { user_id, .. } => Some(*user_id),
@@ -129,9 +132,8 @@ struct MatchEventTypeVisitor;
 impl<'de> Visitor<'de> for MatchEventTypeVisitor {
     type Value = MatchEventType;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(
-            f,
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(
             r#""match-created", "player-joined", "player-left", "match-disbanded", "host-changed", or "other""#
         )
     }
@@ -184,8 +186,8 @@ struct MatchEventVisitor;
 impl<'de> Visitor<'de> for MatchEventVisitor {
     type Value = MatchEvent;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "MatchEvent enum")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("MatchEvent enum")
     }
 
     fn visit_map<A: MapAccess<'de>>(self, mut map: A) -> Result<Self::Value, A::Error> {
@@ -293,7 +295,8 @@ pub struct MatchGame {
     pub scoring_type: ScoringType,
     pub team_type: TeamType,
     pub mods: GameMods,
-    /// Optional in case the map was deleted
+    /// [`BeatmapCompact`](crate::model::beatmap::BeatmapCompact) of the game;
+    /// `None` if the map was deleted
     #[serde(rename = "beatmap")]
     pub map: Option<BeatmapCompact>,
     pub scores: Vec<MatchScore>,
@@ -423,12 +426,14 @@ pub struct MatchList {
 }
 
 impl MatchList {
+    /// Returns whether there is a next page of match results,
+    /// retrievable via [`get_next`](MatchList::get_next).
     #[inline]
     pub fn has_more(&self) -> bool {
         self.cursor.is_some()
     }
 
-    /// If `has_more()` is true, the API can provide the next set of matches and this method will request them.
+    /// If [`has_more`](MatchList::has_more) is true, the API can provide the next set of matches and this method will request them.
     /// Otherwise, this method returns `None`.
     #[inline]
     pub async fn get_next(&self, osu: &Osu) -> Option<OsuResult<MatchList>> {
@@ -444,6 +449,7 @@ pub struct MatchListParams {
 
 #[derive(Clone, Debug, PartialEq, Serialize)]
 pub struct MatchScore {
+    /// Accuracy between `0.0` and `100.0`
     #[serde(serialize_with = "deflate_acc")]
     pub accuracy: f32,
     pub max_combo: u32,
@@ -462,8 +468,8 @@ struct MatchScoreVisitor;
 impl<'de> Visitor<'de> for MatchScoreVisitor {
     type Value = MatchScore;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "a MatchScore struct")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("a MatchScore struct")
     }
 
     fn visit_map<A: MapAccess<'de>>(self, mut map: A) -> Result<Self::Value, A::Error> {
@@ -570,7 +576,7 @@ struct MatchUsersVisitor;
 impl<'de> Visitor<'de> for MatchUsersVisitor {
     type Value = MatchUsers;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a sequence containing UserCompact")
     }
 
@@ -624,7 +630,7 @@ fn serialize_match_users<S: Serializer>(
 impl OsuMatch {
     /// Iterate over references of the match's [`MatchGame`]s.
     #[inline]
-    pub fn games(&self) -> MatchGameIter {
+    pub fn games(&self) -> MatchGameIter<'_> {
         MatchGameIter::new(self.events.iter())
     }
 
@@ -735,7 +741,7 @@ impl OsuMatch {
     /// assert!(osu_match.events.is_empty());
     /// ```
     #[inline]
-    pub fn drain_games(&mut self) -> MatchGameDrain {
+    pub fn drain_games(&mut self) -> MatchGameDrain<'_> {
         MatchGameDrain::new(self.events.drain(..))
     }
 
@@ -797,8 +803,8 @@ struct OsuMatchVisitor;
 impl<'de> Visitor<'de> for OsuMatchVisitor {
     type Value = OsuMatch;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "an OsuMatch struct")
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("an OsuMatch struct")
     }
 
     fn visit_map<A: MapAccess<'de>>(self, mut map: A) -> Result<Self::Value, A::Error> {
@@ -947,10 +953,8 @@ struct BoolVisitor;
 impl<'de> Visitor<'de> for BoolVisitor {
     type Value = Bool;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
-        formatter.write_str(
-            "a bool, a stringified bool, or 0 or 1 in either number, string or char format",
-        )
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("a bool, a stringified bool, or 0 or 1 in either number, string or char format")
     }
 
     fn visit_u64<E: Error>(self, v: u64) -> Result<Self::Value, E> {
