@@ -6,7 +6,10 @@ use super::{
     },
     GameMode,
 };
-use crate::{Osu, OsuResult};
+use crate::{
+    model::user::{CountryCode, Username},
+    Osu, OsuResult,
+};
 
 use chrono::{DateTime, Utc};
 use serde::{
@@ -18,32 +21,41 @@ use std::fmt;
 
 #[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
 pub struct ChartRankings {
+    /// The list of beatmaps in the requested spotlight for the given mode
     #[serde(rename = "beatmapsets")]
     pub mapsets: Vec<Beatmapset>,
     #[serde(
         deserialize_with = "deserialize_user_stats_vec",
         serialize_with = "serialize_user_stats_vec"
     )]
+    /// Score details ordered by score in descending order.
     pub ranking: Vec<UserCompact>,
+    /// Spotlight details
     pub spotlight: Spotlight,
 }
 
 #[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
 pub struct CountryRanking {
+    /// Active user count
     pub active_users: u32,
+    /// Country name
     #[serde(deserialize_with = "deserialize_country")]
     pub country: String,
     #[serde(rename = "code")]
-    pub country_code: String,
+    pub country_code: CountryCode,
+    /// Summed playcount for all users
     #[serde(rename = "play_count")]
     pub playcount: u64,
+    /// Summed performance points for all users
     #[serde(rename = "performance")]
     pub pp: f32,
+    /// Summed ranked score for all users
     pub ranked_score: u64,
 }
 
 #[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
 pub struct CountryRankings {
+    /// The next page of the ranking
     #[serde(
         default,
         rename = "cursor",
@@ -51,6 +63,7 @@ pub struct CountryRankings {
         skip_serializing_if = "Option::is_none"
     )]
     pub next_page: Option<u32>,
+    /// Country details ordered by pp in descending order.
     pub ranking: Vec<CountryRanking>,
     /// Total amount of countries
     pub total: u32,
@@ -91,7 +104,7 @@ struct UserStatsVecVisitor;
 impl<'de> Visitor<'de> for UserStatsVecVisitor {
     type Value = Vec<UserCompact>;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a vec of UserStatistics structs")
     }
 
@@ -119,7 +132,7 @@ struct UserStatsVisitor;
 impl<'de> Visitor<'de> for UserStatsVisitor {
     type Value = UserCompact;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("a UserStatistics struct")
     }
 
@@ -274,7 +287,7 @@ impl<'u> Serialize for UserCompactBorrowed<'u> {
 #[derive(Serialize)]
 struct UserCompactWithoutStats<'u> {
     pub avatar_url: &'u String,
-    pub country_code: &'u String,
+    pub country_code: &'u CountryCode,
     pub default_group: &'u String,
     pub is_active: &'u bool,
     pub is_bot: &'u bool,
@@ -288,7 +301,7 @@ struct UserCompactWithoutStats<'u> {
     pub profile_color: &'u Option<String>,
     #[serde(rename = "id")]
     pub user_id: &'u u32,
-    pub username: &'u String,
+    pub username: &'u Username,
 
     #[serde(skip_serializing_if = "Option::is_none")]
     pub account_history: &'u Option<Vec<AccountHistory>>,
@@ -329,8 +342,6 @@ struct UserCompactWithoutStats<'u> {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub is_nat: &'u Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub is_restricted: &'u Option<bool>,
-    #[serde(skip_serializing_if = "Option::is_none")]
     pub is_silenced: &'u Option<bool>,
     #[serde(
         rename = "loved_beatmapset_count",
@@ -344,7 +355,7 @@ struct UserCompactWithoutStats<'u> {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub page: &'u Option<UserPage>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub previous_usernames: &'u Option<Vec<String>>,
+    pub previous_usernames: &'u Option<Vec<Username>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub rank_history: &'u Option<Vec<u32>>,
     #[serde(
@@ -401,7 +412,6 @@ impl<'u> UserCompactWithoutStats<'u> {
             is_limited_bn,
             is_moderator,
             is_nat,
-            is_restricted,
             is_silenced,
             loved_mapset_count,
             medals,
@@ -449,7 +459,6 @@ impl<'u> UserCompactWithoutStats<'u> {
             is_limited_bn,
             is_moderator,
             is_nat,
-            is_restricted,
             is_silenced,
             loved_mapset_count,
             medals,
@@ -488,7 +497,7 @@ pub(crate) enum RankingType {
 }
 
 impl fmt::Display for RankingType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let kind = match self {
             Self::Charts => "charts",
             Self::Country => "country",
@@ -524,7 +533,7 @@ struct RankingsCursorVisitor;
 impl<'de> Visitor<'de> for RankingsCursorVisitor {
     type Value = Option<u32>;
 
-    fn expecting(&self, f: &mut fmt::Formatter) -> std::fmt::Result {
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> std::fmt::Result {
         f.write_str("a u32, a map containing a `page` field, or null")
     }
 
@@ -562,17 +571,25 @@ fn deserialize_rankings_cursor<'de, D: Deserializer<'de>>(d: D) -> Result<Option
     d.deserialize_option(RankingsCursorVisitor)
 }
 
+/// The details of a spotlight.
 #[derive(Clone, Debug, Deserialize, Serialize)]
 pub struct Spotlight {
+    /// The end date of the spotlight.
     pub end_date: DateTime<Utc>,
+    /// If the spotlight has different mades specific to each [`GameMode`](crate::model::GameMode).
     pub mode_specific: bool,
+    /// The name of the spotlight.
     pub name: String,
+    /// The number of users participating in this spotlight. This is only shown when viewing a single spotlight.
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub participant_count: Option<u32>,
+    /// The ID of this spotlight.
     #[serde(rename = "id")]
     pub spotlight_id: u32,
+    /// The type of spotlight.
     #[serde(rename = "type")]
     pub spotlight_type: String,
+    /// The starting date of the spotlight.
     pub start_date: DateTime<Utc>,
 }
 
