@@ -7,13 +7,14 @@ use crate::{
         score::{BeatmapScores, BeatmapUserScore, Score},
         Cursor, GameMode, GameMods,
     },
+    prelude::{BeatmapCompact, Beatmaps},
     request::{Pending, Query, Request},
     routing::Route,
     Osu,
 };
 
 use futures::future::TryFutureExt;
-use std::fmt::Write;
+use std::{fmt::Write, mem};
 
 #[cfg(feature = "cache")]
 use super::UserId;
@@ -40,6 +41,7 @@ impl<'a> GetBeatmap<'a> {
         }
     }
 
+    /// Specify a beatmap checksum
     #[inline]
     pub fn checksum(mut self, checksum: impl Into<String>) -> Self {
         self.checksum.replace(checksum.into());
@@ -47,6 +49,7 @@ impl<'a> GetBeatmap<'a> {
         self
     }
 
+    /// Specify a beatmap filename
     #[inline]
     pub fn filename(mut self, filename: impl Into<String>) -> Self {
         self.filename.replace(filename.into());
@@ -54,6 +57,7 @@ impl<'a> GetBeatmap<'a> {
         self
     }
 
+    /// Specify a beatmap id
     #[inline]
     pub fn map_id(mut self, map_id: u32) -> Self {
         self.map_id.replace(map_id);
@@ -87,6 +91,52 @@ impl<'a> GetBeatmap<'a> {
 
 poll_req!(GetBeatmap => Beatmap);
 
+/// Get a vec of [`BeatmapCompact`](crate::model::beatmap::BeatmapCompact) by their map ids.
+#[must_use = "futures do nothing unless you `.await` or poll them"]
+pub struct GetBeatmaps<'a> {
+    fut: Option<Pending<'a, Vec<BeatmapCompact>>>,
+    osu: &'a Osu,
+    query: Query,
+}
+
+impl<'a> GetBeatmaps<'a> {
+    #[inline]
+    pub(crate) fn new<I>(osu: &'a Osu, map_ids: I) -> Self
+    where
+        I: IntoIterator<Item = u32>,
+    {
+        let mut query = Query::new();
+
+        for map_id in map_ids.into_iter().take(50) {
+            query.push("ids[]", map_id);
+        }
+
+        Self {
+            fut: None,
+            osu,
+            query,
+        }
+    }
+
+    fn start(&mut self) -> Pending<'a, Vec<BeatmapCompact>> {
+        #[cfg(feature = "metrics")]
+        self.osu.metrics.beatmaps.inc();
+
+        let query = mem::take(&mut self.query);
+        let req = Request::with_query(Route::GetBeatmaps, query);
+
+        let fut = self
+            .osu
+            .inner
+            .request::<Beatmaps>(req)
+            .map_ok(|maps| maps.maps);
+
+        Box::pin(fut)
+    }
+}
+
+poll_req!(GetBeatmaps => Vec<BeatmapCompact>);
+
 /// Get top scores of a beatmap by its id in form of a
 /// vec of [`Score`](crate::model::score::Score)s.
 #[must_use = "futures do nothing unless you `.await` or poll them"]
@@ -117,6 +167,7 @@ impl<'a> GetBeatmapScores<'a> {
         }
     }
 
+    /// Specify the mode of the scores
     #[inline]
     pub fn mode(mut self, mode: GameMode) -> Self {
         self.mode.replace(mode);
@@ -124,6 +175,7 @@ impl<'a> GetBeatmapScores<'a> {
         self
     }
 
+    /// Specify the mods of the scores
     #[deprecated = "Does not currently work since the API requires osu!supporter for this feature"]
     #[inline]
     pub fn mods(mut self, mods: GameMods) -> Self {
@@ -132,6 +184,7 @@ impl<'a> GetBeatmapScores<'a> {
         self
     }
 
+    /// Specify the score type of the scores
     #[deprecated = "Does not currently work since the API requires osu!supporter for this feature"]
     #[inline]
     pub fn score_type(mut self, score_type: &'static str) -> Self {
@@ -202,6 +255,9 @@ poll_req!(GetBeatmapScores => Vec<Score>);
 
 /// Get [`BeatmapUserScore`](crate::model::score::BeatmapUserScore)
 /// of a user on a beatmap by the user's and the map's id.
+///
+/// Note that the contained score will be the user's play on the map
+/// with the most **score** across all mods, not pp.
 #[must_use = "futures do nothing unless you `.await` or poll them"]
 pub struct GetBeatmapUserScore<'a> {
     fut: Option<Pending<'a, BeatmapUserScore>>,
@@ -244,6 +300,7 @@ impl<'a> GetBeatmapUserScore<'a> {
         }
     }
 
+    /// Specify the mode
     #[inline]
     pub fn mode(mut self, mode: GameMode) -> Self {
         self.mode.replace(mode);
@@ -251,6 +308,7 @@ impl<'a> GetBeatmapUserScore<'a> {
         self
     }
 
+    /// Specify the mods
     #[inline]
     pub fn mods(mut self, mods: GameMods) -> Self {
         self.mods.replace(mods);
@@ -371,7 +429,7 @@ poll_req!(GetBeatmapsetEvents => BeatmapsetEvents);
 /// - status: has leaderboard (ranked, loved, approved, and qualified)
 /// - genre: any
 /// - language: any
-/// - extra: does neither contain have video nor storyboard
+/// - extra: does neither contain "have video" nor "have storyboard"
 /// - nsfw: allowed
 /// - sort: by relevance, descending
 ///
@@ -487,7 +545,7 @@ impl<'a> GetBeatmapsetSearch<'a> {
         self
     }
 
-    /// Specify whether mapsets can have a video, defaults to false.
+    /// Specify whether mapsets can have a video, defaults to `false`.
     #[inline]
     pub fn video(mut self, video: bool) -> Self {
         self.video = video;
@@ -495,7 +553,7 @@ impl<'a> GetBeatmapsetSearch<'a> {
         self
     }
 
-    /// Specify whether mapsets can have a storyboard, defaults to false.
+    /// Specify whether mapsets can have a storyboard, defaults to `false`.
     #[inline]
     pub fn storyboard(mut self, storyboard: bool) -> Self {
         self.storyboard = storyboard;
@@ -503,7 +561,7 @@ impl<'a> GetBeatmapsetSearch<'a> {
         self
     }
 
-    /// Specify whether mapsets can be NSFW, defaults to true.
+    /// Specify whether mapsets can be NSFW, defaults to `true`.
     #[inline]
     pub fn nsfw(mut self, nsfw: bool) -> Self {
         self.nsfw = nsfw;
@@ -511,6 +569,7 @@ impl<'a> GetBeatmapsetSearch<'a> {
         self
     }
 
+    /// Specify how the result should be sorted
     #[inline]
     pub fn sort(mut self, sort: BeatmapsetSearchSort, descending: bool) -> Self {
         self.sort.replace(sort);
@@ -625,3 +684,38 @@ impl<'a> GetBeatmapsetSearch<'a> {
 }
 
 poll_req!(GetBeatmapsetSearch => BeatmapsetSearchResult);
+
+/// Get a [`Score`](crate::model::score::Score) struct.
+#[must_use = "futures do nothing unless you `.await` or poll them"]
+pub struct GetScore<'a> {
+    fut: Option<Pending<'a, Score>>,
+    osu: &'a Osu,
+    mode: GameMode,
+    score_id: u64,
+}
+
+impl<'a> GetScore<'a> {
+    #[inline]
+    pub(crate) fn new(osu: &'a Osu, score_id: u64, mode: GameMode) -> Self {
+        Self {
+            fut: None,
+            osu,
+            mode,
+            score_id,
+        }
+    }
+
+    fn start(&mut self) -> Pending<'a, Score> {
+        #[cfg(feature = "metrics")]
+        self.osu.metrics.score.inc();
+
+        let route = Route::GetScore {
+            mode: self.mode,
+            score_id: self.score_id,
+        };
+
+        Box::pin(self.osu.inner.request(Request::new(route)))
+    }
+}
+
+poll_req!(GetScore => Score);
