@@ -24,7 +24,6 @@ mod beatmap;
 mod comments;
 mod forum;
 mod matches;
-mod multiplayer;
 mod news;
 mod ranking;
 mod seasonal_backgrounds;
@@ -35,7 +34,6 @@ pub use beatmap::*;
 pub use comments::*;
 pub use forum::*;
 pub use matches::*;
-pub use multiplayer::*;
 pub use news::*;
 pub use ranking::*;
 pub use seasonal_backgrounds::*;
@@ -79,20 +77,11 @@ impl Request {
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub(crate) struct Query {
     query: String,
 }
 
-impl Default for Query {
-    #[inline]
-    fn default() -> Self {
-        Self {
-            query: String::new(),
-        }
-    }
-}
-
 impl Query {
     #[inline]
     pub(crate) fn new() -> Self {
@@ -112,11 +101,12 @@ impl Query {
 }
 
 impl Display for Query {
-    fn fmt(&self, f: &mut Formatter) -> Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
         if self.query.is_empty() {
             return Ok(());
         }
 
-        write!(f, "?{}", &self.query[..self.query.len() - 1])
+        f.write_char('?')?;
+        f.write_str(&self.query[..self.query.len() - 1])
     }
 }
