@@ -1,6 +1,7 @@
 use crate::{
     model::{
         ranking::{ChartRankings, CountryRankings, RankingType, Rankings, Spotlight},
+        user::CountryCode,
         GameMode,
     },
     request::{Pending, Query, Request},
@@ -95,6 +96,7 @@ impl<'a> GetCountryRankings<'a> {
         }
     }
 
+    /// Specify a page
     #[inline]
     pub fn page(mut self, page: u32) -> Self {
         self.page.replace(page);
@@ -133,7 +135,7 @@ pub struct GetPerformanceRankings<'a> {
     fut: Option<Pending<'a, Rankings>>,
     osu: &'a Osu,
     mode: GameMode,
-    country: Option<String>,
+    country: Option<CountryCode>,
     variant: Option<&'static str>,
     page: Option<u32>,
 }
@@ -153,7 +155,7 @@ impl<'a> GetPerformanceRankings<'a> {
 
     /// Specify a country code.
     #[inline]
-    pub fn country(mut self, country: impl Into<String>) -> Self {
+    pub fn country(mut self, country: impl Into<CountryCode>) -> Self {
         self.country.replace(country.into());
 
         self
