@@ -8,6 +8,7 @@ use crate::{
         user::{User, UserCompact},
         GameMode,
     },
+    prelude::Username,
     request::{Pending, Query, Request},
     routing::Route,
     Osu,
@@ -19,16 +20,23 @@ use std::fmt;
 #[cfg(feature = "cache")]
 use futures::future::TryFutureExt;
 
-// osu! usernames are at most 15 ASCII characters long
-pub(crate) type Username = SmallString<[u8; 15]>;
-
 /// Either a user id as u32 or a username as String.
 ///
-/// Since usernames will be stored as `String`, if possible,
-/// make use of `From<String>` instead of `From<&String>`.
-#[derive(Debug)]
+/// Use the `From` implementations to create this enum
+///
+/// # Example
+///
+/// ```
+/// use rosu_v2::request::UserId;
+///
+/// let user_id: UserId = 123_456.into();
+/// let user_id: UserId = "my username".into();
+/// ```
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub enum UserId {
+    /// Represents a user through their user id
     Id(u32),
+    /// Represents a user through their username
     Name(Username),
 }
 
@@ -61,7 +69,7 @@ impl From<String> for UserId {
 }
 
 impl fmt::Display for UserId {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Id(id) => write!(f, "{}", id),
             Self::Name(name) => f.write_str(name),
@@ -69,6 +77,49 @@ impl fmt::Display for UserId {
     }
 }
 
+/// Get the [`User`](crate::model::user::User) of the authenticated user.
+///
+/// Note that the client has to be initialized with the `identify` scope
+/// through the OAuth process in order for this endpoint to not return an error.
+///
+/// See [`OsuBuilder::with_authorization`](crate::OsuBuilder::with_authorization).
+#[must_use = "futures do nothing unless you `.await` or poll them"]
+pub struct GetOwnData<'a> {
+    fut: Option<Pending<'a, User>>,
+    osu: &'a Osu,
+    mode: Option<GameMode>,
+}
+
+impl<'a> GetOwnData<'a> {
+    #[inline]
+    pub(crate) fn new(osu: &'a Osu) -> Self {
+        Self {
+            fut: None,
+            osu,
+            mode: None,
+        }
+    }
+
+    /// Specify the mode for which the user data should be retrieved
+    #[inline]
+    pub fn mode(mut self, mode: GameMode) -> Self {
+        self.mode.replace(mode);
+
+        self
+    }
+
+    fn start(&mut self) -> Pending<'a, User> {
+        #[cfg(feature = "metrics")]
+        self.osu.metrics.own_data.inc();
+
+        let req = Request::new(Route::GetOwnData { mode: self.mode });
+
+        Box::pin(self.osu.inner.request(req))
+    }
+}
+
+poll_req!(GetOwnData => User);
+
 /// Get a [`User`](crate::model::user::User) by their id.
 #[must_use = "futures do nothing unless you `.await` or poll them"]
 pub struct GetUser<'a> {
@@ -89,6 +140,7 @@ impl<'a> GetUser<'a> {
         }
     }
 
+    /// Specify the mode for which the user data should be retrieved
     #[inline]
     pub fn mode(mut self, mode: GameMode) -> Self {
         self.mode.replace(mode);
@@ -127,13 +179,12 @@ poll_req!(GetUser => User);
 /// Get the [`Beatmapset`](crate::model::beatmap::Beatmapset)s of a user by their id.
 ///
 /// If no map type specified, either manually through
-/// [`map_type`](crate::request::GetUserBeatmapsets::map_type),
+/// [`status`](crate::request::GetUserBeatmapsets::status),
 /// or through any of the methods [`loved`](crate::request::GetUserBeatmapsets::loved),
-/// [`favourite`](crate::request::GetUserBeatmapsets::favourite),
 /// [`graveyard`](crate::request::GetUserBeatmapsets::graveyard),
-/// [`ranked_and_approved`](crate::request::GetUserBeatmapsets::ranked_and_approved),
-/// [`unranked`](crate::request::GetUserBeatmapsets::unranked),
-/// it defaults to `ranked_and_approved`.
+/// [`ranked`](crate::request::GetUserBeatmapsets::ranked),
+/// [`pending`](crate::request::GetUserBeatmapsets::pending),
+/// it defaults to `ranked`.
 #[must_use = "futures do nothing unless you `.await` or poll them"]
 pub struct GetUserBeatmapsets<'a> {
     fut: Option<Pending<'a, Vec<Beatmapset>>>,
@@ -176,6 +227,7 @@ impl<'a> GetUserBeatmapsets<'a> {
         }
     }
 
+    /// Limit the amount of results in the response
     #[inline]
     pub fn limit(mut self, limit: usize) -> Self {
         self.limit.replace(limit);
@@ -183,6 +235,8 @@ impl<'a> GetUserBeatmapsets<'a> {
         self
     }
 
+    /// Set an offset for the requested elements
+    /// e.g. skip the first `offset` amount in the list
     #[inline]
     pub fn offset(mut self, offset: usize) -> Self {
         self.offset.replace(offset);
@@ -190,7 +244,9 @@ impl<'a> GetUserBeatmapsets<'a> {
         self
     }
 
-    pub fn map_type(mut self, map_type: RankStatus) -> Self {
+    /// Only include mapsets with the specified status
+    #[inline]
+    pub fn status(mut self, map_type: RankStatus) -> Self {
         self.map_type = match map_type {
             RankStatus::Approved | RankStatus::Ranked => "ranked",
             RankStatus::Graveyard => "graveyard",
@@ -316,6 +372,7 @@ impl<'a> GetUserKudosu<'a> {
         }
     }
 
+    /// Limit the amount of results in the response
     #[inline]
     pub fn limit(mut self, limit: usize) -> Self {
         self.limit.replace(limit);
@@ -323,6 +380,8 @@ impl<'a> GetUserKudosu<'a> {
         self
     }
 
+    /// Set an offset for the requested elements
+    /// e.g. skip the first `offset` amount in the list
     #[inline]
     pub fn offset(mut self, offset: usize) -> Self {
         self.offset.replace(offset);
@@ -418,6 +477,8 @@ impl<'a> GetUserMostPlayed<'a> {
         self
     }
 
+    /// Set an offset for the requested elements
+    /// e.g. skip the first `offset` amount in the list
     #[inline]
     pub fn offset(mut self, offset: usize) -> Self {
         self.offset.replace(offset);
@@ -515,6 +576,7 @@ impl<'a> GetRecentEvents<'a> {
         }
     }
 
+    /// Limit the amount of results in the response
     #[inline]
     pub fn limit(mut self, limit: usize) -> Self {
         self.limit.replace(limit);
@@ -522,6 +584,8 @@ impl<'a> GetRecentEvents<'a> {
         self
     }
 
+    /// Set an offset for the requested elements
+    /// e.g. skip the first `offset` amount in the list
     #[inline]
     pub fn offset(mut self, offset: usize) -> Self {
         self.offset.replace(offset);
@@ -574,14 +638,16 @@ poll_req!(GetRecentEvents => Vec<RecentEvent>);
 pub(crate) enum ScoreType {
     Best,
     First,
+    Pinned,
     Recent,
 }
 
 impl fmt::Display for ScoreType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let kind = match self {
             Self::Best => "best",
             Self::First => "firsts",
+            Self::Pinned => "pinned",
             Self::Recent => "recent",
         };
 
@@ -643,7 +709,7 @@ impl<'a> GetUserScores<'a> {
         }
     }
 
-    /// The API provides at most 51 results per requests.
+    /// The API provides at most 100 results per requests.
     #[inline]
     pub fn limit(mut self, limit: usize) -> Self {
         self.limit.replace(limit);
@@ -651,6 +717,8 @@ impl<'a> GetUserScores<'a> {
         self
     }
 
+    /// Set an offset for the requested elements
+    /// e.g. skip the first `offset` amount in the list
     #[inline]
     pub fn offset(mut self, offset: usize) -> Self {
         self.offset.replace(offset);
@@ -658,6 +726,7 @@ impl<'a> GetUserScores<'a> {
         self
     }
 
+    /// Specify the mode of the scores
     #[inline]
     pub fn mode(mut self, mode: GameMode) -> Self {
         self.mode.replace(mode);
@@ -665,6 +734,9 @@ impl<'a> GetUserScores<'a> {
         self
     }
 
+    /// Specify whether failed scores can be included.
+    ///
+    /// Only relevant for [`recent`](GetUserScores::recent)
     #[inline]
     pub fn include_fails(mut self, include_fails: bool) -> Self {
         self.include_fails.replace(include_fails);
@@ -688,6 +760,14 @@ impl<'a> GetUserScores<'a> {
         self
     }
 
+    /// Get the pinned scores of a user.
+    #[inline]
+    pub fn pinned(mut self) -> Self {
+        self.score_type = ScoreType::Pinned;
+
+        self
+    }
+
     /// Get recent scores of a user.
     #[inline]
     pub fn recent(mut self) -> Self {
@@ -701,6 +781,7 @@ impl<'a> GetUserScores<'a> {
         match self.score_type {
             ScoreType::Best => self.osu.metrics.user_top_scores.inc(),
             ScoreType::First => self.osu.metrics.user_first_scores.inc(),
+            ScoreType::Pinned => self.osu.metrics.user_pinned_scores.inc(),
             ScoreType::Recent => self.osu.metrics.user_recent_scores.inc(),
         }
 
