@@ -11,6 +11,7 @@ use std::{borrow::Cow, fmt::Write};
 #[non_exhaustive]
 pub(crate) enum Route {
     GetBeatmap,
+    GetBeatmaps,
     GetBeatmapScores {
         map_id: u32,
     },
@@ -30,26 +31,12 @@ pub(crate) enum Route {
     GetMatch {
         match_id: Option<u32>,
     },
-    #[allow(dead_code)]
-    GetMultiplayerScore {
-        room: u32,
-        playlist: u32,
-        score_id: u32,
-    },
-    #[allow(dead_code)]
-    GetMultiplayerScores {
-        room: u32,
-        playlist: u32,
-    },
-    #[allow(dead_code)]
-    GetMultiplayerUserHighScore {
-        room: u32,
-        playlist: u32,
-        user_id: u32,
-    },
     GetNews {
         news: Option<()>,
     },
+    GetOwnData {
+        mode: Option<GameMode>,
+    },
     GetRankings {
         mode: GameMode,
         ranking_type: RankingType,
@@ -57,6 +44,10 @@ pub(crate) enum Route {
     GetRecentEvents {
         user_id: u32,
     },
+    GetScore {
+        mode: GameMode,
+        score_id: u64,
+    },
     GetSeasonalBackgrounds,
     GetSpotlights,
     GetUser {
@@ -87,6 +78,7 @@ impl Route {
     pub(crate) fn into_parts(self) -> (Method, Cow<'static, str>) {
         match self {
             Self::GetBeatmap => (Method::GET, "beatmaps/lookup".into()),
+            Self::GetBeatmaps => (Method::GET, "beatmaps".into()),
             Self::GetBeatmapScores { map_id } => {
                 (Method::GET, format!("beatmaps/{}/scores", map_id).into())
             }
@@ -111,30 +103,6 @@ impl Route {
 
                 (Method::GET, path)
             }
-            Self::GetMultiplayerScore {
-                room,
-                playlist,
-                score_id,
-            } => (
-                Method::GET,
-                format!("rooms/{}/playlist/{}/scores/{}", room, playlist, score_id).into(),
-            ),
-            Self::GetMultiplayerScores { room, playlist } => (
-                Method::GET,
-                format!("rooms/{}/playlist/{}/scores", room, playlist).into(),
-            ),
-            Self::GetMultiplayerUserHighScore {
-                room,
-                playlist,
-                user_id,
-            } => {
-                let path = format!(
-                    "rooms/{}/playlist/{}/scores/users/{}",
-                    room, playlist, user_id
-                );
-
-                (Method::GET, path.into())
-            }
             Self::GetNews { news } => {
                 let path = match news {
                     Some(_news) => unimplemented!(),
@@ -143,6 +111,14 @@ impl Route {
 
                 (Method::GET, path)
             }
+            Self::GetOwnData { mode } => {
+                let path = match mode {
+                    Some(mode) => format!("me/{}", mode).into(),
+                    None => "me".into(),
+                };
+
+                (Method::GET, path)
+            }
             Self::GetRankings { mode, ranking_type } => (
                 Method::GET,
                 format!("rankings/{}/{}", mode, ranking_type).into(),
@@ -151,6 +127,9 @@ impl Route {
                 Method::GET,
                 format!("users/{}/recent_activity", user_id).into(),
             ),
+            Self::GetScore { mode, score_id } => {
+                (Method::GET, format!("scores/{}/{}", mode, score_id).into())
+            }
             Self::GetSeasonalBackgrounds => (Method::GET, "seasonal-backgrounds".into()),
             Self::GetSpotlights => (Method::GET, "spotlights".into()),
             Self::GetUser { user_id, mode } => {
