@@ -1,5 +1,7 @@
 extern crate rosu_v2;
 
+use std::{env, error::Error, fmt::Write};
+
 use dotenv::dotenv;
 use once_cell::sync::OnceCell;
 use rosu_v2::{
@@ -9,7 +11,6 @@ use rosu_v2::{
     },
     Osu,
 };
-use std::{env, error::Error};
 
 #[cfg(feature = "cache")]
 use rosu_v2::model::GameMods;
@@ -32,6 +33,7 @@ macro_rules! unwind_error {
 
 static OSU: OnceCell<Osu> = OnceCell::new();
 
+// Be sure you pass `--test-threads=1` to `cargo test` when running
 async fn init() {
     if OSU.get().is_none() {
         let _ = env_logger::builder().is_test(true).try_init();
@@ -49,48 +51,33 @@ async fn init() {
             .client_secret(client_secret)
             .build()
             .await
-            .unwrap_or_else(|err| {
-                panic!(
-                    "failed to build osu! client:\n  - caused by: {}:\n  - caused by: {}",
-                    err,
-                    err.source().map_or_else(
-                        || format!("No source"),
-                        |e| format!(
-                            "{}\n  - caused by: {}",
-                            e,
-                            e.source()
-                                .map_or_else(|| format!("No source"), |e| e.to_string())
-                        )
-                    )
-                )
+            .unwrap_or_else(|e| {
+                let mut output = format!("failed to build osu! client:\n  - caused by: {}", e);
+                let mut err: &dyn Error = &e;
+
+                while let Some(src) = err.source() {
+                    let _ = Write::write_fmt(&mut output, format_args!("\n  - caused by: {}", src));
+                    err = src;
+                }
+
+                panic!("{}", output)
             });
 
         OSU.set(osu).unwrap_or_else(|_| panic!("failed to set OSU"));
     }
 }
 
-macro_rules! get_id {
-    ($name:ident, $id:literal) => {
-        fn $name() -> u32 {
-            $id
-        }
-    };
-}
+const ADESSO_BALLA: u32 = 171024;
+const BREEZEBLOCKS: u32 = 3187415;
 
-// Map id
-get_id!(adesso_balla, 171024);
+const HIKOUI_GUMO: u32 = 357161;
 
-// Mapset id
-get_id!(hikoui_gumo, 357161);
+const BADEWANNE3: u32 = 2211396;
+const SYLAS: u32 = 3906405;
 
-// Player id
-get_id!(badewanne3, 2211396);
+const DE_VS_CA: u32 = 71028303;
 
-// Mapper id
-get_id!(sylas, 3906405);
-
-// Match id
-get_id!(de_vs_ca, 71028303);
+const COOKIEZI_FREEDOM_DIVE: u64 = 2177560145;
 
 fn osu() -> &'static Osu {
     OSU.get().expect("OSU not initialized")
@@ -101,17 +88,17 @@ fn osu() -> &'static Osu {
 async fn custom() {
     init().await;
 
-    let req_fut = osu().user(sylas());
+    let req_fut = osu().user(SYLAS);
 
     let result = req_fut.await.unwrap();
-    println!("Result 1:\n{:?}", result);
+    println!("Result:\n{:#?}", result);
 }
 
 #[tokio::test]
 async fn beatmap() {
     init().await;
 
-    match osu().beatmap().map_id(adesso_balla()).await {
+    match osu().beatmap().map_id(ADESSO_BALLA).await {
         Ok(map) => println!(
             "Received {} - {}",
             map.mapset.as_ref().unwrap().artist,
@@ -124,11 +111,24 @@ async fn beatmap() {
     }
 }
 
+#[tokio::test]
+async fn beatmaps() {
+    init().await;
+
+    match osu().beatmaps([ADESSO_BALLA, BREEZEBLOCKS]).await {
+        Ok(maps) => println!("Received {} maps", maps.len()),
+        Err(why) => {
+            unwind_error!(error, why, "Error while requesting beatmap: {}");
+            panic!()
+        }
+    }
+}
+
 #[tokio::test]
 async fn beatmap_scores() {
     init().await;
 
-    match osu().beatmap_scores(adesso_balla()).await {
+    match osu().beatmap_scores(ADESSO_BALLA).await {
         Ok(scores) => println!("Received {} scores", scores.len()),
         Err(why) => {
             unwind_error!(error, why, "Error while requesting beatmap scores: {}");
@@ -143,7 +143,7 @@ async fn beatmap_user_score() {
     init().await;
 
     match osu()
-        .beatmap_user_score(adesso_balla(), badewanne3())
+        .beatmap_user_score(ADESSO_BALLA, BADEWANNE3)
         .mods(GameMods::Hidden | GameMods::HardRock | GameMods::HalfTime)
         .await
     {
@@ -166,7 +166,7 @@ async fn beatmap_user_score() {
 async fn beatmapset() {
     init().await;
 
-    match osu().beatmapset(hikoui_gumo()).await {
+    match osu().beatmapset(HIKOUI_GUMO).await {
         Ok(mapset) => println!("Received mapset with {} maps", mapset.maps.unwrap().len()),
         Err(why) => {
             unwind_error!(println, why, "Error while requesting beatmapset: {}");
@@ -299,7 +299,7 @@ async fn recent_events() {
 async fn kudosu() {
     init().await;
 
-    match osu().kudosu(sylas()).limit(5).offset(1).await {
+    match osu().kudosu(SYLAS).limit(5).offset(1).await {
         Ok(history) => {
             let sum: i32 = history.iter().map(|entry| entry.amount).sum();
 
@@ -312,48 +312,6 @@ async fn kudosu() {
     }
 }
 
-#[tokio::test]
-#[ignore = "TODO"]
-async fn multiplayer_score() {
-    init().await;
-
-    // match osu().multiplayer_score(room, playlist, score_id).await {
-    //     Ok(score) => todo!(),
-    //     Err(why) => {
-    //         unwind_error!(error, why, "Error while requesting score: {}");
-    //         panic!()
-    //     }
-    // }
-}
-
-#[tokio::test]
-#[ignore = "TODO"]
-async fn multiplayer_scores() {
-    init().await;
-
-    // match osu().multiplayer_scores(room, playlist).await {
-    //     Ok(scores) => todo!(),
-    //     Err(why) => {
-    //         unwind_error!(error, why, "Error while requesting scores: {}");
-    //         panic!()
-    //     }
-    // }
-}
-
-#[tokio::test]
-#[ignore = "TODO"]
-async fn multiplayer_user_highscore() {
-    init().await;
-
-    // match osu().multiplayer_user_highscore(room, playlist, user_id).await {
-    //     Ok(score) => todo!(),
-    //     Err(why) => {
-    //         unwind_error!(error, why, "Error while requesting user highscore: {}");
-    //         panic!()
-    //     }
-    // }
-}
-
 #[tokio::test]
 async fn news() {
     init().await;
@@ -371,7 +329,7 @@ async fn news() {
 async fn osu_match() {
     init().await;
 
-    match osu().osu_match(de_vs_ca()).await {
+    match osu().osu_match(DE_VS_CA).await {
         Ok(osu_match) => {
             println!(
                 "Received match, got {} events and {} users",
@@ -399,6 +357,23 @@ async fn osu_matches() {
     }
 }
 
+#[tokio::test]
+#[ignore = "requires OAuth to not throw an error"]
+async fn own_data() {
+    init().await;
+
+    match osu().own_data().mode(GameMode::TKO).await {
+        Ok(user) => println!(
+            "Received own data showing a last activity of {:?}",
+            user.last_visit
+        ),
+        Err(why) => {
+            unwind_error!(error, why, "Error while requesting own data: {}");
+            panic!()
+        }
+    }
+}
+
 #[tokio::test]
 async fn performance_rankings() {
     init().await;
@@ -422,6 +397,22 @@ async fn performance_rankings() {
     }
 }
 
+#[tokio::test]
+async fn score() {
+    init().await;
+
+    match osu().score(COOKIEZI_FREEDOM_DIVE, GameMode::STD).await {
+        Ok(score) => println!(
+            "Received {}'s FREEDOM DIVE score",
+            score.user.unwrap().username
+        ),
+        Err(why) => {
+            unwind_error!(error, why, "Error while requesting score: {}");
+            panic!()
+        }
+    }
+}
+
 #[tokio::test]
 async fn score_rankings() {
     init().await;
@@ -496,7 +487,7 @@ async fn user_beatmapsets() {
     init().await;
 
     match osu()
-        .user_beatmapsets(sylas())
+        .user_beatmapsets(SYLAS)
         .limit(5)
         .ranked()
         .offset(2)
@@ -514,12 +505,7 @@ async fn user_beatmapsets() {
 async fn user_most_played() {
     init().await;
 
-    match osu()
-        .user_most_played(badewanne3())
-        .limit(5)
-        .offset(2)
-        .await
-    {
+    match osu().user_most_played(BADEWANNE3).limit(5).offset(2).await {
         Ok(scores) => println!(
             "Received {} scores, the first is map id {}",
             scores.len(),
@@ -540,12 +526,12 @@ async fn user_scores() {
     match osu()
         .user_scores("Badewanne3")
         .mode(GameMode::CTB)
-        .limit(10)
+        .limit(99)
         .offset(1)
         .best()
         .await
     {
-        Ok(scores) => println!("Received {} scores", scores.len()),
+        Ok(scores) => assert_eq!(scores.len(), 99),
         Err(why) => {
             unwind_error!(error, why, "Error while requesting user scores: {}");
             panic!()
@@ -559,7 +545,7 @@ async fn users() {
     init().await;
 
     #[allow(deprecated)]
-    match osu().users(&[badewanne3(), sylas()]).await {
+    match osu().users(&[BADEWANNE3, SYLAS]).await {
         Ok(users) => println!("Received {} users", users.len()),
         Err(why) => {
             unwind_error!(error, why, "Error while requesting users: {}");
