@@ -27,7 +27,7 @@ fn get_country_ranking() -> CountryRanking {
     CountryRanking {
         active_users: 2,
         country: "belgiania".to_owned(),
-        country_code: "be".to_owned(),
+        country_code: "be".into(),
         playcount: 420,
         pp: 123.45,
         ranked_score: 1_000_000_000_000_000,
@@ -106,7 +106,7 @@ fn get_mapset() -> Beatmapset {
         converts: Some(vec![]),
         covers: get_mapset_covers(),
         creator: Some(get_user_compact()),
-        creator_name: "god".to_owned(),
+        creator_name: "god".into(),
         creator_id: 2,
         description: Some("description".to_owned()),
         discussion_enabled: true,
@@ -199,7 +199,7 @@ fn get_mapset_compact() -> BeatmapsetCompact {
         artist: "artist".to_owned(),
         artist_unicode: Some("äöü".to_owned()),
         covers: get_mapset_covers(),
-        creator_name: "god".to_owned(),
+        creator_name: "god".into(),
         creator_id: 2,
         favourite_count: 1_234_567,
         genre: Some(Genre::Rock),
@@ -395,7 +395,7 @@ fn get_mapset_events() -> BeatmapsetEvents {
                     map_id: 123,
                     version: "epic version".to_owned(),
                     new_user_id: 98,
-                    new_username: "new name".to_owned(),
+                    new_username: "new name".into(),
                 },
                 created_at: get_date(),
                 user_id: 99,
@@ -560,7 +560,7 @@ fn get_user() -> User {
         avatar_url: String::new(),
         comments_count: 0,
         country: "belgiania".to_owned(),
-        country_code: "be".to_owned(),
+        country_code: "be".into(),
         cover: UserCover {
             custom_url: Some(String::new()),
             url: String::new(),
@@ -591,12 +591,11 @@ fn get_user() -> User {
         forum_post_count: 0,
         profile_color: Some(String::new()),
         profile_order: vec![ProfilePage::Me, ProfilePage::TopRanks],
-        skype: Some(String::new()),
         title: Some(String::new()),
         title_url: Some(String::new()),
         twitter: Some(String::new()),
         user_id: 12345,
-        username: "bob".to_owned(),
+        username: "bob".into(),
         website: Some(String::new()),
         account_history: Some(vec![AccountHistory {
             id: Some(1),
@@ -615,14 +614,15 @@ fn get_user() -> User {
         follower_count: Some(2),
         graveyard_mapset_count: Some(8),
         groups: Some(vec![Group {
+            color: Some("#FFFFFF".to_owned()),
+            description: Some("epic group".to_owned()),
+            has_modes: true,
             id: 1,
             identifier: String::new(),
             is_probationary: true,
+            modes: Some(vec![GameMode::STD, GameMode::MNA]),
             name: "group".to_owned(),
             short_name: "g".to_owned(),
-            description: Some("epic group".to_owned()),
-            color: Some("#FFFFFF".to_owned()),
-            modes: Some(vec![GameMode::STD, GameMode::MNA]),
         }]),
         is_admin: Some(true),
         is_bng: Some(false),
@@ -631,7 +631,6 @@ fn get_user() -> User {
         is_limited_bn: Some(true),
         is_moderator: Some(true),
         is_nat: Some(true),
-        is_restricted: Some(true),
         is_silenced: Some(true),
         loved_mapset_count: Some(3),
         mapping_follower_count: Some(5),
@@ -643,7 +642,7 @@ fn get_user() -> User {
             html: String::new(),
             raw: String::new(),
         }),
-        previous_usernames: Some(vec!["b0b".to_owned()]),
+        previous_usernames: Some(vec!["b0b".into()]),
         rank_history: Some(vec![50, 40, 30, 35]),
         ranked_mapset_count: Some(800),
         replays_watched_counts: Some(vec![MonthlyCount {
@@ -666,7 +665,7 @@ fn get_user() -> User {
 fn get_user_compact() -> UserCompact {
     UserCompact {
         avatar_url: String::new(),
-        country_code: "be".to_owned(),
+        country_code: "be".into(),
         default_group: "default".to_owned(),
         is_active: true,
         is_bot: false,
@@ -677,7 +676,7 @@ fn get_user_compact() -> UserCompact {
         pm_friends_only: false,
         profile_color: Some("#FFFFFF".to_owned()),
         user_id: 12345,
-        username: "bob".to_owned(),
+        username: "bob".into(),
         account_history: Some(vec![AccountHistory {
             id: Some(1),
             history_type: HistoryType::Note,
@@ -701,14 +700,15 @@ fn get_user_compact() -> UserCompact {
         follower_count: Some(2),
         graveyard_mapset_count: Some(34),
         groups: Some(vec![Group {
+            color: Some("#FFFFFF".to_owned()),
+            description: Some("epic group".to_owned()),
+            has_modes: true,
             id: 1,
             identifier: String::new(),
             is_probationary: true,
+            modes: Some(vec![GameMode::STD, GameMode::MNA]),
             name: "group".to_owned(),
             short_name: "g".to_owned(),
-            description: Some("epic group".to_owned()),
-            color: Some("#FFFFFF".to_owned()),
-            modes: Some(vec![GameMode::STD, GameMode::MNA]),
         }]),
         is_admin: Some(true),
         is_bng: Some(false),
@@ -717,7 +717,6 @@ fn get_user_compact() -> UserCompact {
         is_limited_bn: Some(true),
         is_moderator: Some(false),
         is_nat: Some(false),
-        is_restricted: Some(false),
         is_silenced: Some(false),
         loved_mapset_count: Some(34),
         medals: Some(vec![MedalCompact {
@@ -732,7 +731,7 @@ fn get_user_compact() -> UserCompact {
             html: String::new(),
             raw: String::new(),
         }),
-        previous_usernames: Some(vec!["b0b".to_owned()]),
+        previous_usernames: Some(vec!["b0b".into()]),
         rank_history: Some(vec![50, 40, 30, 35]),
         ranked_mapset_count: Some(34),
         replays_watched_counts: Some(vec![MonthlyCount {
